#*************************************************************************/
# dtktoracadm.py
# DESCRIPTION:
# Main file for dtktoracadm converter utility.
# 
# To convert DTK scripts to equivalent RACADM scripts , customers can optionally use this conversion tool on their scripts. 
# This tool :
# -	Takes the existing script as input file 
# -	Parses the same and maps the DTK commands to equivalent RACADM command
# -	Maps the input options to equivalent RACADM options.
# -	Generates the new output file with the equivalent RACADM commands.
# -	Create a log file detailing the changed and unchanged commands in the input scripts.
# AUTHOR   : Anuj Kumar Verma < anuj_k_verma@dell.com >
# REVIEWER : HE, Ramesha <ramesha_he@dell.com>
# NOTES:
# 
# DELL COMPUTER CORPORATION PROPRIETARY INFORMATION
# This software is supplied under the terms of a license agreement or
# non-disclosure agreement with Dell Computer Corporation and may not
# be copied or disclosed except in accordance with the terms of that
# agreement.
# Copyright (c) Dell Computer 2015.  All rights reserved.
#*************************************************************************/

#!/usr/bin/python
import sys,  re , ast , os


#code for replacing strings which are different in syscfg and racadm 
def wordReplace(command_text):
	dict = {
		"all":"All",
		"optimizermode":"OptimizerMode",
		"sparemode":"SpareMode",
		"mirrormode":"MirrorMode",
		"adveccmode":"AdvEccMode",
		"sparewithadveccmode":"SpareWithAdvEccMode",
		"nominal":"Nominal",
		"level1":"Level1",
		"level2":"Level2",
		"maxdatarate":"MaxDataRate",
		"administrator":"Administrator",
		"basic":"Basic",
		"bootmode": "BootMode",
		"bios": "Bios",
		"capenable":"Cap.Enable",
		"connectionmode":"ConnectionMode",
		"disable": "Disabled",
		"dnsserver1v6":"DNS1",
		"dnsserver2v6":"DNS2",
		"enable":"Enabled",
		"enabled":"Enabled",
		"encryptkey":"EncryptionKey",
		"gateway":"Gateway",
		"gatewayv6":"Gateway",
		"ipaddress":"Address",
		"ipv6address1":"Address1",
		"ipv6address2":"Address2",
		"linklocaladdrv6":"LinkLocalAddress",
		"maxpowercap":"Cap.MaxThreshold",
		"msgcommbitrate":"BaudRate",
		"minpowercap":"Cap.MinThreshold",
		"msgcommflowctrl":"FlowControl",
		"noflowctrl":"None",
		"off": "Off",
		"on": "On",
		"operator": "Operator",
		"prefixlengthv6":"PrefixLength",
		"rtscts":"RTS/CTS",
		"solbitrate":"BaudRate",
		"solcharaccuminterval":"AccumulateInterval",
		"solcharsendthreshold":"SendThreshold",
		"solprivlevel":"MinPrivilege",
		"subnetmask":"Netmask",
		"terminal":"Terminal",
		"tmcfghandshakectrl":"HandshakeControl",
		"tmcfglineediting":"LineEdit",
		"tmcfgnewlineseq":"NewLineSeq",
		"uefi": "Uefi",
		"user": "User"
		}
	r_comment = re.compile(r"^(#|::)")
	comments = r_comment.findall(str(command_text.strip()))
	if(len(comments) != 0 ):
		return command_text
	
	for key, val in dict.iteritems():
		r = re.compile(r"(?:^|\s|\"|\'|=)%s(?:$|\s|\"|\'|\n|=)" %key)
		match_pattern = r.findall(str(command_text))
		for pattern in match_pattern:
			replace_pattern = re.sub(key,val,pattern)
			command_text = re.sub(pattern,replace_pattern,str(command_text));
	return command_text


#Function to build equivalent racadm command from syscfg command using dictionary mappings  if no equivalent command found return None
def buildRacadmCommand(sysCfgCommand):
	sysargList = sysCfgCommand.split()
	if(debug):
		print ('\tList of syscfg tokens : '+str(sysargList))
	
	allreplaced = True
	racargList = []
	
	#if there are no subtokens then leave the command as it is 
	hasSubCommand = False
	for sysarg in sysargList:
		if '-' in sysarg or '--' in sysarg:
			hasSubCommand = True
			break
	
	if hasSubCommand == False:
		return None
	
		
	#if there are more than one subtokens then we do not convert since no equivalent racadm command to process multiple arguments
	if (( len(sysargList)) == 3 and (sysargList[2] == 2 or sysargList[2] == 1)):
		if (len(sysargList)) > 3 or (('-' in sysargList[1] or '--' in sysargList[1]) and  (len(sysargList) > 2)):
			if(debug):
					print ("Length of subcommands more than one for : " + str(sysargList))
			return None
	
	if ('-' not in sysargList[1] and '--' not in sysargList[1]):
		sysargList[1:] = [' '.join(sysargList[1:])]
		
		
	for sysarg in sysargList:
		# for handling commands having '=' we need to separate LHS and RHS 
		if '=' in sysarg:
			strlhsrhs = [] 
			strlhsrhs = sysarg.split('=')
			cmd = dtktoracadm_dict.get((strlhsrhs[0]+'=').lower())
			if cmd is None: 
				if(debug):
					print ("No equivalent racadm token found for : " + strlhsrhs[0])
				allreplaced = False
				break;
			if(debug):
				print ("\tFor the syscfg token : " + strlhsrhs[0] + " equivalent racadm token : "+cmd)
			racarg = cmd + " " + wordReplace(strlhsrhs[1])
		else:
			racarg = dtktoracadm_dict.get(sysarg.lower())
			if(debug):
				if(racarg):
					print ("\tFor the syscfg token : " + sysarg + " equivalent racadm token : "+racarg)
				else:
					print ("\tFor the syscfg token :" + sysarg + " no equivalent racadm token  found ")
		
		if racarg is None:
			allreplaced = False
			break;
		else:
			racargList.append(racarg)
			
	if allreplaced: 
		racadmCmd =  ' '.join(racargList)
		racadmCmd = wordReplace(racadmCmd)
		if(debug):
			print ("\tEquivalent racadm command formed : " + racadmCmd)
		return racadmCmd
	else:
		return None
	
# function to extract lines having syscfg command from input file 
def extractCommands(list_file , inputfile):
	for file_name in list_file:
		if(file_name == ""):
			continue
		try:
			fh_read = open(file_name,"r")
		except IOError:
			print ("\"" + file_name + "\"" +' - script does not exist. Please correct' +"\"" + inputfile +"\"" 'and re-run the converter.')
			continue
			
		file_name_write = file_name[file_name.rfind('/')+1:]
		print("\nProcessing file " + file_name_write)
		fh_write = open("racscripts//"+file_name[file_name.rfind('/')+1:] , "w")

		lines = fh_read.readlines()
				
		# definition for convertCommand : function to convert syscfg command to racadm command using buildRacadmCommand
		def convertCommand(syscommand):
			#regular expression used to find DTK commands
			sysCmd1Obj = re.search(r"(syscfg)\s+([a-zA-Z0-9])*\s*((-{1,2})[a-zA-Z0-9_=\s\.,]+)*",syscommand)
			returncmd = "invalid"
			
			if(debug):
				if(sysCmd1Obj):
					print("---------------------------------------------------------")
					print("Search result : " + sysCmd1Obj.group())
			
			if(sysCmd1Obj):
				sysCmd1Str = sysCmd1Obj.group()
				sysCmd1Prefix = sysCmd1Str[0:sysCmd1Str.find('syscfg')]
				sysCmd1Str = sysCmd1Str[sysCmd1Str.find('syscfg'):]
				racadmCommand = buildRacadmCommand(sysCmd1Str)
				
				if racadmCommand is None:
					returncmd = syscommand
				else:
					if(debug):
						print("\tBuilt racadm command : " + racadmCommand)

					returncmd = syscommand[0:sysCmd1Obj.start()] + sysCmd1Prefix + racadmCommand + convertCommand(syscommand[sysCmd1Obj.end():]) + "\n"
					
			else:
				returncmd = syscommand
			
			if(sysCmd1Obj):
				if(returncmd == syscommand):
					fh_unchanged.write("No equivalent RACADM command available ==> " + syscommand.strip("\n") + "in " + file_name+ "\n\n")
				else:	
					fh_changed.write("Syscfg command changed From ==> " + syscommand.strip("\n") + " To RACADM command ==> " 
					+ returncmd.strip("\n") + " in " + file_name+ "\n\n")
			return returncmd

				
				
		cmdLines = list(map(convertCommand , lines))
		

		
		cmdLines = list(map(wordReplace , cmdLines))		
		
		for cmd in cmdLines:
			fh_write.write(cmd)
			
		fh_read.close()
		fh_write.close()
		
def convertor(inputfile):
	try:
		fh_ip_filename = open(inputfile, "r")
	except IOError:
		print(inputfile + '-Input file not present')
		sys.exit()
		
	list_file = fh_ip_filename.read().split('\n')
	if(debug):
		print 'List of script files to be converted : ' , list_file , "\n"
	fh_ip_filename.close()
	extractCommands(list_file , inputfile)
	
if __name__ == "__main__":
	list_file_name = sys.argv
	debug = 0
	inputfile = 'linux_script.lst' 
	if len(sys.argv) > 1:
		if( len(sys.argv) == 3 ):
			if(list_file_name[1] == "debug"):
				print ("Usage : python dtktoracadm.py <filename> <option>")
				print ("Options : debug")
				sys.exit()
			else :
				inputfile = list_file_name[1]
				if( list_file_name[2] == "debug"):
					debug = 1
		else:
			if(list_file_name[1] == "debug"):
				debug = 1
				inputfile = 'linux_script.lst'
			else:
				inputfile = list_file_name[1]

	if(debug):		
		print ('Input file :'+ inputfile)

	fh_changed = open("racscripts//changed_tokens.log","w")
	fh_unchanged = open("racscripts//unchanged_tokens.log","w")
	
	try:
		fh_dict = open("dtktoracadm_dict.map","r")
	except IOError:
		print ('Dictionary not found , check (dtktoracadm_dict.map) in CONVERTOR folder')
		sys.exit()
		
	rawdictlist = fh_dict.read().split('\n')
	
	rawdict = ''.join(rawdictlist)
	dict = rawdict[rawdict.find('dictionary='):]
	dict = dict[dict.find('{'):]
	dict = dict[:dict.find('}')+1]
	dtktoracadm_dict = ast.literal_eval(str(dict));
	convertor(inputfile)
	os.system('chmod +x racscripts/*');